@@ -44,6 +44,7 @@
 #include "dialogs/certificateselectiondialog.h"
 #include "commands/detailscommand.h"
 
+#include <Libkleo/KeyCache>
 #include <Libkleo/KeyFilter>
 #include <Libkleo/KeyListModel>
 #include <Libkleo/KeyListSortFilterProxyModel>
@@ -54,12 +55,52 @@
 
 #include <gpgme++/key.h>
 
+#include <QGpgME/KeyForMailboxJob>
+#include <QGpgME/Protocol>
+
 using namespace Kleo;
 using namespace Kleo::Dialogs;
 using namespace GpgME;
 
 Q_DECLARE_METATYPE(GpgME::Key)
 
+static QStringList s_lookedUpKeys;
+
+namespace
+{
+class ProxyModel : public KeyListSortFilterProxyModel
+{
+    Q_OBJECT
+
+public:
+    ProxyModel(QObject *parent = nullptr)
+        : KeyListSortFilterProxyModel(parent)
+    {
+    }
+
+    QVariant data(const QModelIndex &index, int role) const override
+    {
+        if (!index.isValid()) {
+            return QVariant();
+        }
+
+        switch (role) {
+        case Qt::DecorationRole: {
+            const auto key = KeyListSortFilterProxyModel::data(index,
+                    Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+            Q_ASSERT(!key.isNull());
+            if (key.isNull()) {
+                return QVariant();
+            }
+            return Kleo::Formatting::iconForUid(key.userID(0));
+        }
+        default:
+            return KeyListSortFilterProxyModel::data(index, role);
+        }
+    }
+};
+} // namespace
+
 CertificateLineEdit::CertificateLineEdit(AbstractKeyListModel *model,
                                          QWidget *parent,
                                          KeyFilter *filter)
@@ -77,7 +118,7 @@ CertificateLineEdit::CertificateLineEdit(AbstractKeyListModel *model,
     QFontMetrics fm(font());
 
     auto *completer = new QCompleter(this);
-    auto *completeFilterModel = new KeyListSortFilterProxyModel(completer);
+    auto *completeFilterModel = new ProxyModel(completer);
     completeFilterModel->setKeyFilter(mFilter);
     completeFilterModel->setSourceModel(model);
     completer->setModel(completeFilterModel);
@@ -91,7 +132,7 @@ CertificateLineEdit::CertificateLineEdit(AbstractKeyListModel *model,
         mFilterModel->setKeyFilter(mFilter);
     }
 
-    connect(model, &QAbstractItemModel::dataChanged,
+    connect(KeyCache::instance().get(), &Kleo::KeyCache::keyListingDone,
             this, &CertificateLineEdit::updateKey);
     connect(this, &QLineEdit::editingFinished,
             this, &CertificateLineEdit::updateKey);
@@ -99,7 +140,8 @@ CertificateLineEdit::CertificateLineEdit(AbstractKeyListModel *model,
             this, &CertificateLineEdit::editChanged);
     connect(mLineAction, &QAction::triggered,
             this, &CertificateLineEdit::dialogRequested);
-
+    connect(this, &QLineEdit::editingFinished, this,
+            &CertificateLineEdit::checkLocate);
     updateKey();
 
     /* Take ownership of the model to prevent double deletion when the
@@ -117,6 +159,24 @@ void CertificateLineEdit::editChanged()
     mEditFinished = false;
 }
 
+void CertificateLineEdit::checkLocate()
+{
+    if (!key().isNull()) {
+        // Already have a key
+        return;
+    }
+
+    // Only check once per mailbox
+    const auto mailText = text();
+    if (s_lookedUpKeys.contains(mailText)) {
+        return;
+    }
+    s_lookedUpKeys << mailText;
+    qCDebug(KLEOPATRA_LOG) << "Lookup job for" << mailText;
+    QGpgME::KeyForMailboxJob *job = QGpgME::openpgp()->keyForMailboxJob();
+    job->start(mailText);
+}
+
 void CertificateLineEdit::updateKey()
 {
     const auto mailText = text();
@@ -206,3 +266,11 @@ bool CertificateLineEdit::isEmpty() const
 {
     return text().isEmpty();
 }
+
+void CertificateLineEdit::setKeyFilter(const std::shared_ptr<KeyFilter> &filter)
+{
+    mFilter = filter;
+    mFilterModel->setKeyFilter(filter);
+}
+
+#include "certificatelineedit.moc"
