@@ -1,4 +1,5 @@
 /*  Copyright (c) 2016 Klarälvdalens Datakonsult AB
+                  2017 Intevation GmbH
 
     Kleopatra is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,23 +19,31 @@
 #include "certificatedetailswidget.h"
 #include "ui_certificatedetailswidget.h"
 #include "kleopatra_debug.h"
+#include "exportdialog.h"
 #include "trustchainwidget.h"
 #include "subkeyswidget.h"
+#include "weboftrustdialog.h"
 
 #include "commands/changepassphrasecommand.h"
 #include "commands/changeexpirycommand.h"
 #include "commands/certifycertificatecommand.h"
 #include "commands/adduseridcommand.h"
 #include "commands/genrevokecommand.h"
+#include "commands/detailscommand.h"
 #include "commands/dumpcertificatecommand.h"
 
 #include <libkleo/formatting.h>
 #include <libkleo/dn.h>
 #include <libkleo/keycache.h>
 
+#include <gpgme++/context.h>
 #include <gpgme++/key.h>
+#include <gpgme++/keylistresult.h>
 #include <gpgme++/tofuinfo.h>
 
+#include <QGpgME/Protocol>
+#include <QGpgME/KeyListJob>
+
 #include <QToolTip>
 #include <QDateTime>
 #include <QDialogButtonBox>
@@ -48,6 +57,8 @@
 
 Q_DECLARE_METATYPE(GpgME::UserID)
 
+using namespace Kleo;
+
 class CertificateDetailsWidget::Private
 {
 public:
@@ -61,6 +72,9 @@ public:
 
     void revokeUID(const GpgME::UserID &uid);
     void genRevokeCert();
+    void certifyClicked();
+    void webOfTrustClicked();
+    void exportClicked();
     void addUserID();
     void changePassphrase();
     void changeExpiration();
@@ -74,8 +88,14 @@ public:
 
     void smimeLinkActivated(const QString &link);
 
+    void setUpdatedKey(const GpgME::Key &key);
+    void keyListDone(const GpgME::KeyListResult &,
+                     const std::vector<GpgME::Key> &, const QString &,
+                     const GpgME::Error &);
+
     Ui::CertificateDetailsWidget ui;
     GpgME::Key key;
+    bool updateInProgress;
 private:
     CertificateDetailsWidget *q;
 };
@@ -92,8 +112,12 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
 
     ui.changePassphraseBtn->setVisible(hasSecret);
     ui.genRevokeBtn->setVisible(isOpenPGP && hasSecret);
+    ui.certifyBtn->setVisible(isOpenPGP && !hasSecret);
     ui.changeExpirationBtn->setVisible(isOpenPGP && hasSecret);
     ui.addUserIDBtn->setVisible(hasSecret);
+    ui.webOfTrustBtn->setVisible(isOpenPGP);
+
+    ui.hboxLayout_1->addStretch(1);
 
     ui.validFrom->setText(Kleo::Formatting::creationDateString(key));
     const QString expiry = Kleo::Formatting::expirationDateString(key);
@@ -109,12 +133,9 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
 
     ui.userIDTable->clear();
 
-    QStringList headers = { i18n("Email"), i18n("Name") };
-    if (isOpenPGP) {
-        headers << i18n("Trust Level");
-        if (canRevokeUID) {
-            headers << QString();
-        }
+    QStringList headers = { i18n("Email"), i18n("Name"), i18n("Trust Level") };
+    if (canRevokeUID) {
+        headers << QString();
     }
     ui.userIDTable->setColumnCount(headers.count());
     ui.userIDTable->setColumnWidth(0, 200);
@@ -122,16 +143,34 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
     ui.userIDTable->setHeaderLabels(headers);
 
     const auto uids = key.userIDs();
-    for (const auto &uid : uids) {
+    for (unsigned int i = 0; i < uids.size(); ++i) {
+        const auto &uid = uids[i];
         auto item = new QTreeWidgetItem;
         const QString toolTip = tofuTooltipString(uid);
         item->setData(0, Qt::UserRole, QVariant::fromValue(uid));
-        item->setData(0, Qt::DisplayRole, Kleo::Formatting::prettyEMail(uid));
+
+        auto pMail = Kleo::Formatting::prettyEMail(uid);
+        if (!isOpenPGP && pMail.isEmpty()) {
+            // S/MIME UserIDs are sometimes split, with one userID
+            // containing the name another the Mail, we merge these
+            // UID's into a single item.
+
+            if (i + 1 < uids.size()) {
+                pMail = Kleo::Formatting::prettyEMail(uids[i + 1]);
+                // skip next uid
+                ++i;
+            }
+        }
+        item->setData(0, Qt::DisplayRole, pMail);
         item->setData(0, Qt::ToolTipRole, toolTip);
         item->setData(1, Qt::DisplayRole, Kleo::Formatting::prettyName(uid));
         item->setData(1, Qt::ToolTipRole, toolTip);
-        if (isOpenPGP) {
-            QIcon trustIcon;
+
+        QIcon trustIcon;
+        if (updateInProgress) {
+           trustIcon = QIcon::fromTheme(QStringLiteral("emblem-question"));
+           item->setData(2, Qt::DisplayRole, i18n("Updating..."));
+        } else {
             switch (uid.validity()) {
             case GpgME::UserID::Unknown:
             case GpgME::UserID::Undefined:
@@ -148,23 +187,21 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
                 trustIcon = QIcon::fromTheme(QStringLiteral("emblem-success"));
                 break;
             }
-            item->setData(2, Qt::DecorationRole, trustIcon);
             item->setData(2, Qt::DisplayRole, Kleo::Formatting::validityShort(uid));
-            item->setData(2, Qt::ToolTipRole, toolTip);
+        }
+        item->setData(2, Qt::DecorationRole, trustIcon);
+        item->setData(2, Qt::ToolTipRole, toolTip);
 
-            ui.userIDTable->addTopLevelItem(item);
+        ui.userIDTable->addTopLevelItem(item);
 
-            if (canRevokeUID) {
-                auto button = new QPushButton;
-                button->setIcon(QIcon::fromTheme(QStringLiteral("entry-delete")));
-                button->setToolTip(i18n("Revoke this User ID"));
-                button->setMaximumWidth(32);
-                QObject::connect(button, &QPushButton::clicked,
-                                q, [this, uid]() { revokeUID(uid); });
-                ui.userIDTable->setItemWidget(item, 4, button);
-            }
-        } else {
-            ui.userIDTable->addTopLevelItem(item);
+        if (canRevokeUID) {
+            auto button = new QPushButton;
+            button->setIcon(QIcon::fromTheme(QStringLiteral("entry-delete")));
+            button->setToolTip(i18n("Revoke this User ID"));
+            button->setMaximumWidth(32);
+            QObject::connect(button, &QPushButton::clicked,
+                            q, [this, uid]() { revokeUID(uid); });
+            ui.userIDTable->setItemWidget(item, 4, button);
         }
     }
 }
@@ -208,6 +245,31 @@ void CertificateDetailsWidget::Private::genRevokeCert()
     cmd->start();
 }
 
+void CertificateDetailsWidget::Private::certifyClicked()
+{
+    auto cmd = new Kleo::Commands::CertifyCertificateCommand(key);
+    QObject::connect(cmd, &Kleo::Commands::CertifyCertificateCommand::finished,
+                     q, [this, cmd]() {
+                         ui.certifyBtn->setEnabled(true);
+                     });
+    ui.certifyBtn->setEnabled(false);
+    cmd->start();
+}
+
+void CertificateDetailsWidget::Private::webOfTrustClicked()
+{
+    QScopedPointer<WebOfTrustDialog> dlg(new WebOfTrustDialog(q));
+    dlg->setKey(key);
+    dlg->exec();
+}
+
+void CertificateDetailsWidget::Private::exportClicked()
+{
+    QScopedPointer<ExportDialog> dlg(new ExportDialog(q));
+    dlg->setKey(key);
+    dlg->exec();
+}
+
 void CertificateDetailsWidget::Private::addUserID()
 {
     auto cmd = new Kleo::Commands::AddUserIDCommand(key);
@@ -223,7 +285,7 @@ void CertificateDetailsWidget::Private::keysMayHaveChanged()
 {
     auto newKey = Kleo::KeyCache::instance()->findByFingerprint(key.primaryFingerprint());
     if (!newKey.isNull()) {
-        q->setKey(newKey);
+        setUpdatedKey(newKey);
     }
 }
 
@@ -336,6 +398,30 @@ void CertificateDetailsWidget::Private::setupPGPProperties()
             q, [this](const QPoint &p) { userIDTableContextMenuRequested(p); });
 }
 
+static QString formatDNToolTip(const Kleo::DN &dn)
+{
+    QString html = QStringLiteral("<table border=\"0\" cell-spacing=15>");
+
+    const auto appendRow = [&html, dn](const QString &lbl, const QString &attr) {
+                                const QString val = dn[attr];
+                                if (!val.isEmpty()) {
+                                    html += QStringLiteral(
+                                            "<tr><th style=\"text-align: left; white-space: nowrap\">%1:</th>"
+                                                "<td style=\"white-space: nowrap\">%2</td>"
+                                            "</tr>").arg(lbl, val);
+                                }
+                            };
+    appendRow(i18n("Common Name"), QStringLiteral("CN"));
+    appendRow(i18n("Organization"), QStringLiteral("O"));
+    appendRow(i18n("Street"), QStringLiteral("STREET"));
+    appendRow(i18n("City"),  QStringLiteral("L"));
+    appendRow(i18n("State"), QStringLiteral("ST"));
+    appendRow(i18n("Country"), QStringLiteral("C"));
+    html += QStringLiteral("</table>");
+
+    return html;
+}
+
 void CertificateDetailsWidget::Private::setupSMIMEProperties()
 {
     HIDE_ROW(publishing)
@@ -355,53 +441,36 @@ void CertificateDetailsWidget::Private::setupSMIMEProperties()
     } else {
         owner = i18nc("<name> of <company>", "%1 of %2", name, o);
     }
-    ui.smimeOwner->setText(QStringLiteral("<a href=\"#ownerDetails\">%1</a>").arg(owner));
-    ui.smimeOwner->setContextMenuPolicy(Qt::NoContextMenu);
+    ui.smimeOwner->setText(owner);
+    ui.smimeOwner->setTextInteractionFlags(Qt::TextBrowserInteraction);
 
     const Kleo::DN issuerDN(key.issuerName());
     const QString issuerCN = issuerDN[QStringLiteral("CN")];
     const QString issuer = issuerCN.isEmpty() ? key.issuerName() : issuerCN;
     ui.smimeIssuer->setText(QStringLiteral("<a href=\"#issuerDetails\">%1</a>").arg(issuer));
+
+    ui.smimeIssuer->setToolTip(formatDNToolTip(issuerDN));
+
+    ui.smimeOwner->setToolTip(formatDNToolTip(dn));
+
 }
 
 void CertificateDetailsWidget::Private::smimeLinkActivated(const QString &link)
 {
-    Kleo::DN dn;
-    QWidget *w;
-    if (link == QLatin1String("#ownerDetails")) {
-        dn = Kleo::DN(key.userID(0).id());
-        w = ui.smimeOwner;
-    } else if (link == QLatin1String("#issuerDetails")) {
-        dn = Kleo::DN(key.issuerName());
-        w = ui.smimeIssuer;
-    } else {
-        Q_UNREACHABLE();
+    if (link == QLatin1String("#issuerDetails")) {
+        const auto parentKey = KeyCache::instance()->findIssuers(key, KeyCache::NoOption);
+
+        if (!parentKey.size()) {
+            return;
+        }
+        auto cmd = new Kleo::Commands::DetailsCommand(parentKey[0], nullptr);
+        cmd->setParentWidget(q);
+        cmd->start();
         return;
     }
-
-    QString html = QStringLiteral("<table border=\"0\" cell-spacing=15>");
-    const auto appendRow = [&html, dn](const QString &lbl, const QString &attr) {
-                                const QString val = dn[attr];
-                                if (!val.isEmpty()) {
-                                    html += QStringLiteral(
-                                            "<tr><th style=\"text-align: left; white-space: nowrap\">%1:</th>"
-                                                "<td style=\"white-space: nowrap\">%2</td>"
-                                            "</tr>").arg(lbl, val);
-                                }
-                            };
-    appendRow(i18n("Common Name"), QStringLiteral("CN"));
-    appendRow(i18n("Organization"), QStringLiteral("O"));
-    appendRow(i18n("Street"), QStringLiteral("STREET"));
-    appendRow(i18n("City"),  QStringLiteral("L"));
-    appendRow(i18n("State"), QStringLiteral("ST"));
-    appendRow(i18n("Country"), QStringLiteral("C"));
-    html += QStringLiteral("</table>");
-
-    QToolTip::showText(QCursor::pos(), html, w);
+    qCWarning(KLEOPATRA_LOG) << "Unknown link activated:" << link;
 }
 
-
-
 CertificateDetailsWidget::CertificateDetailsWidget(QWidget *parent)
     : QWidget(parent)
     , d(new Private(this))
@@ -426,6 +495,12 @@ CertificateDetailsWidget::CertificateDetailsWidget(QWidget *parent)
             this, [this]() { d->showMoreDetails(); });
     connect(d->ui.publishing, &QPushButton::pressed,
             this, [this]() { d->publishCertificate(); });
+    connect(d->ui.certifyBtn, &QPushButton::clicked,
+            this, [this]() { d->certifyClicked(); });
+    connect(d->ui.webOfTrustBtn, &QPushButton::clicked,
+            this, [this]() { d->webOfTrustClicked(); });
+    connect(d->ui.exportBtn, &QPushButton::clicked,
+            this, [this]() { d->exportClicked(); });
 
     connect(Kleo::KeyCache::instance().get(), &Kleo::KeyCache::keysMayHaveChanged,
             this, [this]() { d->keysMayHaveChanged(); });
@@ -435,17 +510,53 @@ CertificateDetailsWidget::~CertificateDetailsWidget()
 {
 }
 
+void CertificateDetailsWidget::Private::keyListDone(const GpgME::KeyListResult &,
+                                                    const std::vector<GpgME::Key> &keys,
+                                                    const QString &,
+                                                    const GpgME::Error &) {
+        updateInProgress = false;
+        if (keys.size() != 1) {
+            qCWarning(KLEOPATRA_LOG) << "Invalid keylist result in update.";
+            return;
+        }
+        // As we listen for keysmayhavechanged we get the update
+        // after updating the keycache.
+        KeyCache::mutableInstance()->insert(keys);
+}
+
+void CertificateDetailsWidget::Private::setUpdatedKey(const GpgME::Key &k)
+{
+    key = k;
+
+    setupCommonProperties();
+    if (key.protocol() == GpgME::OpenPGP) {
+        setupPGPProperties();
+    } else {
+        setupSMIMEProperties();
+    }
+}
+
 void CertificateDetailsWidget::setKey(const GpgME::Key &key)
 {
-    d->key = key;
-    d->key.update(); // Fetch TOFU info (TODO: could be blocking, use async?)
-
-    d->setupCommonProperties();
-    if (key.protocol() == GpgME::OpenPGP) {
-        d->setupPGPProperties();
-    } else {
-        d->setupSMIMEProperties();
+    if (key.protocol() == GpgME::CMS) {
+        // For everything but S/MIME this should be quick
+        // and we don't need to show another status.
+        d->updateInProgress = true;
     }
+    d->setUpdatedKey(key);
+
+    // Run a keylistjob with full details (TOFU / Validate)
+    QGpgME::KeyListJob *job = key.protocol() == GpgME::OpenPGP ? QGpgME::openpgp()->keyListJob(false, true, true) :
+                                                                 QGpgME::smime()->keyListJob(false, true, true);
+
+    auto ctx = QGpgME::Job::context(job);
+    ctx->addKeyListMode(GpgME::WithTofu);
+
+    // Windows QGpgME new style connect problem makes this necessary.
+    connect(job, SIGNAL(result(GpgME::KeyListResult, std::vector<GpgME::Key>, QString, GpgME::Error)),
+            this, SLOT(keyListDone(GpgME::KeyListResult, std::vector<GpgME::Key>, QString, GpgME::Error)));
+
+    job->start(QStringList() << key.primaryFingerprint(), key.hasSecret());
 }
 
 GpgME::Key CertificateDetailsWidget::key() const
@@ -501,3 +612,5 @@ GpgME::Key CertificateDetailsDialog::key() const
     Q_ASSERT(w);
     return w->key();
 }
+
+#include "moc_certificatedetailswidget.cpp"
